# Disable relative imports, and remove magical current directory sys.path entry
from __future__ import absolute_import
import sys
if __name__ == "__main__": sys.path.pop(0)

import daemon
import daemon.pidlockfile
import lockfile
import os.path
import os

import flawless.server.server

def main(argv):
  # Process argv
  if len(argv) <= 1:
    print "\nUsage: python flawless_daemon.py FLAWLESS_CONFIG_PATH [PID_FILE_PATH]"
    print "  FLAWLESS_CONFIG_PATH - The path to the flawless.cfg config you want to use"
    print "  PID_FILE_PATH - (optional) The path you want to for the PID lock file\n"
    return
  flawless_cfg_path = os.path.abspath(argv[1])
  pid_file_path = os.path.abspath(argv[2]) if len(argv) == 3 else None

  # Initialize context
  context = daemon.DaemonContext()
  context.detach_process = True
  context.working_directory = '.'

  # Setup logging of output
  pid = os.getpid()
  filename = "flawless-%d.ERROR" % pid
  error_log = open(filename, "w+", 1)
  context.stdout = error_log
  context.stderr = error_log

  # Setup PID file
  if pid_file_path:
    context.pidfile = daemon.pidlockfile.TimeoutPIDLockFile(pid_file_path, acquire_timeout=2)

  try:
    with context:
      os.setpgid(0, os.getpid())
      flawless.server.server.serve(flawless_cfg_path)
  except lockfile.LockTimeout:
    sys.stderr.write("Error: Couldn't acquire lock on %s. Exiting.\n" % pid_file_path)
    sys.exit(1)


if __name__ == '__main__':
  main(sys.argv)



#!/usr/bin/env python
#
# Copyright (c) 2011-2013, Shopkick Inc.
# All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# ---
# Author: John Egan <john@shopkick.com>

import abc
import os.path

import flawless.lib.config
from flawless.lib.data_structures import ProxyContainerMethodsMetaClass
from flawless.lib.data_structures.persistent_dictionary import PersistentDictionary


class StorageInterface(object):
    """By default Flawless stores everything on disk which means there can only be one centralized instance of
    Flawless. You can implement your own instance of StorageInterface that connects to a backend database and pass
    it into flawless.server.server.serve. Then it is possible to have the flawless server be horizontally scalable
    since the database serves as the centralized source of truth.

    It is worth noting is that the keys used in this interface are instances of api_ttypes.ErrorKey and the
    values are instances of api_ttypes.ErrorInfo
    """

    __metaclass__ = abc.ABCMeta

    def __init__(self, week_prefix):
        """week_prefix is a string used to partition keys by week. For instance, with disk storage, we create a new
        file for every unique week_prefix. For a database you may want to consider prepending all keys with
        week_prefix."""
        self.week_prefix = week_prefix

    def open(self):
        """Called to create connection to storage"""
        pass

    def sync(self):
        """Called periodically to flush data"""
        pass

    def close(self):
        """Called to close connection to storage"""
        pass

    @abc.abstractmethod
    def iteritems(self):
        """Should return iterator of tuples (key, value) for all entries for the given self.week_prefix"""
        pass

    @abc.abstractmethod
    def __setitem__(self, key, item):
        pass

    @abc.abstractmethod
    def __getitem__(self, key):
        """Should raise KeyError if key does not exist"""
        pass

    @abc.abstractmethod
    def __contains__(self, key):
        pass


class DiskStorage(object):

    __metaclass__ = ProxyContainerMethodsMetaClass

    def __init__(self, week_prefix):
        self.week_prefix = week_prefix
        config = flawless.lib.config.get()
        filepath = os.path.join(config.data_dir_path, "flawless-errors-", week_prefix)
        self.disk_dict = PersistentDictionary(filepath)

    def _proxyfunc_(attr, self, *args, **kwargs):
        return getattr(self.disk_dict, attr)(*args, **kwargs)

    def open(self):
        self.disk_dict.open()

    def sync(self):
        self.disk_dict.sync()

    def close(self):
        self.disk_dict.close()

    def iteritems(self):
        return self.disk_dict.dict.iteritems()

StorageInterface.register(DiskStorage)



#!/usr/bin/env python
#
# Copyright (c) 2011-2013, Shopkick Inc.
# All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# ---
# Author: John Egan <john@shopkick.com>

from flawless.lib.data_structures import ProxyContainerMethodsMetaClass
from flawless.lib.data_structures.storage import StorageInterface


class StorageStub(object):

    __metaclass__ = ProxyContainerMethodsMetaClass
    _proxyfunc_ = lambda attr, self, *args, **kwargs: getattr(self.dict, attr)(*args, **kwargs)

    def __init__(self, week_prefix):
        self.week_prefix = week_prefix
        self.dict = dict()

    def open(self):
        pass

    def sync(self):
        pass

    def close(self):
        pass

    def iteritems(self):
        return self.dict.iteritems()

StorageInterface.register(StorageStub)



#!/usr/bin/env python
#
# Copyright (c) 2011-2013, Shopkick Inc.
# All rights reserved.
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# ---
# Author: John Egan <john@shopkick.com>

import functools
import inspect

import flawless.client


def wrap_function(func=None, error_threshold=None, reraise_exception=True):
    ''' Wraps a function with reporting to errors backend '''
    # This if/else allows wrap_function to behave like a normal decorator when
    # used like:
    #         @wrap_function
    #         def some_func():
    #
    # However, it also allows wrap_function to also be passed keyword arguments
    # like the following:
    #         @wrap_function(error_threshold=3, reraise_exception=False)
    #         def some_func():
    if func:
        return flawless.client._wrap_function_with_error_decorator(
            func=func, error_threshold=error_threshold, reraise_exception=reraise_exception)
    else:
        return functools.partial(flawless.client._wrap_function_with_error_decorator,
                                 error_threshold=error_threshold,
                                 reraise_exception=reraise_exception)


def wrap_class(cls, error_threshold=None):
    ''' Wraps a class with reporting to errors backend by decorating each function of the class.
            Decorators are injected under the classmethod decorator if they exist.
    '''
    for method_name, method in inspect.getmembers(cls, inspect.ismethod):
        wrapped_method = flawless.client._wrap_function_with_error_decorator(
            method if not method.im_self else method.im_func,
            save_current_stack_trace=False,
            error_threshold=error_threshold,
        )
        if method.im_self:
            wrapped_method = classmethod(wrapped_method)
        setattr(cls, method_name, wrapped_method)
    return cls
